/**
 *  Copyright (C) 2009  Thomas Volk
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package net.sf.gtools.jms

import javax.jms.Session
import javax.jms.Destination
import javax.jms.Message
import javax.jms.MessageProducer
import javax.jms.MessageConsumer
import javax.jms.MessageListener

public class JmsClient {
    private MessageProducer producer
    private MessageConsumer consumer
    private Map param
    private JmsDestination destination
    private Session session
	
    JmsClient(JmsDestination inDest, Map inParam) {
        destination = inDest
        param = inParam
    }

    Session getSession() {
        destination.session
    }
	
    MessageProducer getProducer() {
        if(!producer) {
            producer = destination.createProducer()
        }
        return producer
    }
	
    MessageConsumer getConsumer() {
        if(!consumer) {
            consumer = destination.createConsumer(param)
        }
        return consumer
    }
	
    void send(Message message) {
        getProducer().send(message)
    }
	
    Message receive(Long timeout) {
        return getConsumer().receive(timeout)
    }
	
    Message receive() {
        return getConsumer().receive()
    }
	
    Message receiveNoWait() {
        return getConsumer().receiveNoWait()
    }
	
    void listen(Closure closure) {
        getConsumer().setMessageListener({msg ->
                use(JmsCategory) { closure(msg) }
            } as MessageListener)
    }
}
